@@ -52,6 +52,7 @@ pub enum Field {
     Sgid,
     IsHidden,
     HasXattrs,
+    Capabilities,
     IsShebang,
     IsEmpty,
     Width,
@@ -141,6 +142,7 @@ impl FromStr for Field {
             "sgid" => Ok(Field::Sgid),
             "is_hidden" => Ok(Field::IsHidden),
             "has_xattrs" => Ok(Field::HasXattrs),
+            "capabilities" | "caps" => Ok(Field::Capabilities),
             "is_shebang" => Ok(Field::IsShebang),
             "is_empty" => Ok(Field::IsEmpty),
             "width" => Ok(Field::Width),
@@ -313,4 +315,4 @@ impl Field {
             _ => false
         }
     }
-}
\ No newline at end of file
+}
