@@ -1,12 +1,15 @@
+use std::collections::HashMap;
 use std::fs::File;
 use std::ops::Add;
 use std::ops::Index;
-use std::path::Path;
+use std::path::{Path, PathBuf};
 
 use regex::Captures;
 use regex::Error;
 use regex::Regex;
 
+use crate::util::error_exit;
+
 #[derive(Clone, Debug)]
 pub struct GitignoreFilter {
     pub regex: Regex,
@@ -22,6 +25,66 @@ impl GitignoreFilter {
     }
 }
 
+pub fn search_upstream_gitignore(gitignore_map: &mut HashMap<PathBuf, Vec<GitignoreFilter>>, dir: &Path) {
+    if let Ok(canonical_path) = crate::util::canonical_path(&dir.to_path_buf()) {
+        let mut path = PathBuf::from(canonical_path);
+
+        loop {
+            let parent_found = path.pop();
+
+            if !parent_found {
+                return;
+            }
+
+            update_gitignore_map(gitignore_map, &mut path);
+        }
+    }
+}
+
+pub fn update_gitignore_map(gitignore_map: &mut HashMap<PathBuf, Vec<GitignoreFilter>>, path: &Path) {
+    let gitignore_file = path.join(".gitignore");
+    if gitignore_file.is_file() {
+        let regexes = parse_gitignore(&gitignore_file, &path);
+        gitignore_map.insert(path.to_path_buf(), regexes);
+    }
+}
+
+pub fn get_gitignore_filters(gitignore_map: &mut HashMap<PathBuf, Vec<GitignoreFilter>>, dir: &Path) -> Vec<GitignoreFilter> {
+    let mut result = vec![];
+
+    for (dir_path, regexes) in &mut *gitignore_map {
+        if dir.to_path_buf() == *dir_path {
+            for ref mut rx in regexes {
+                result.push(rx.clone());
+            }
+
+            return result;
+        }
+    }
+
+    let mut path = dir.to_path_buf();
+
+    loop {
+        let parent_found = path.pop();
+
+        if !parent_found {
+            return result;
+        }
+
+        for (dir_path, regexes) in &mut *gitignore_map {
+            if path == *dir_path {
+                let mut tmp = vec![];
+                for ref mut rx in regexes {
+                    tmp.push(rx.clone());
+                }
+                tmp.append(&mut result);
+                result.clear();
+                result.append(&mut tmp);
+            }
+        }
+    }
+}
+
 pub fn matches_gitignore_filter(gitignore_filters: &Option<Vec<GitignoreFilter>>, file_name: &str, is_dir: bool) -> bool {
     match gitignore_filters {
         Some(gitignore_filters) => {
@@ -62,7 +125,7 @@ fn convert_file_name_for_matcher(file_name: &str) -> String {
     }
 }
 
-pub fn parse_gitignore(file_path: &Path, dir_path: &Path) -> Vec<GitignoreFilter> {
+fn parse_gitignore(file_path: &Path, dir_path: &Path) -> Vec<GitignoreFilter> {
     let mut result = vec![];
 
     let git_dir = dir_path.join(".git");
@@ -149,7 +212,7 @@ fn convert_gitignore_glob(glob: &str, file_path: &Path) -> Result<Regex, Error>
             "." => "\\.",
             "*" => "[^/]*",
             "?" => "[^/]+",
-            _ => panic!("Error parsing pattern")
+            _ => error_exit(".gitignore", "Error parsing pattern")
         }.to_string()
     }).to_string();
 
