@@ -1,3 +1,5 @@
+#[cfg(target_os = "linux")]
+pub(crate) mod capabilities;
 mod datetime;
 mod glob;
 pub(crate) mod japanese;
@@ -148,6 +150,11 @@ pub fn error_message(source: &str, description: &str) {
     eprint!("{}: {}", source, description);
 }
 
+pub fn error_exit(source: &str, description: &str) -> ! {
+    error_message(source, description);
+    std::process::exit(2);
+}
+
 pub fn get_extension(s: &str) -> String {
     match Path::new(s).extension() {
         Some(ext) => ext.to_string_lossy().to_string(),
@@ -350,9 +357,7 @@ pub fn format_filesize(size: u64, modifier: &str) -> String {
             fixed_at = None;
             format = humansize::BINARY;
         },
-        _ => {
-            panic!("Unknown file size modifier");
-        }
+        _ => error_exit("Unknown file size modifier", modifier.as_str())
     };
 
     if zeroes == -1 {
@@ -426,6 +431,10 @@ pub fn has_extension(file_name: &str, extensions: &Vec<String>) -> bool {
     false
 }
 
+pub fn looks_like_regexp(s: &str) -> bool {
+    s.contains('*') || s.contains('[') || s.contains('?')
+}
+
 pub fn is_text_mime(mime: &str) -> bool {
     mime.starts_with("text/") ||
     mime.contains("+xml") ||
